<?php
/**
 * Plugin Name: Polylang - Country Detection
 * Description: Detects the preferred language according on the country
 * Version: 1.0.0
 * Requires at least: 4.9
 * Requires PHP: 5.6
 * Author: Markus Froehlich
 * Author URI:  mailto:markusfroehlich01@gmail.com
 * License: GPL v3 or later
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 */

/*
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // don't access directly
};

/**
 * Controls the plugin, as well as activation, and deactivation
 */
class Polylang_Country_Detection
{
    /**
	 * Constructor
	 */
	public function __construct()
    {
	    /*
	     * Plugin initialization
		 * Take no action before all plugins are loaded
	     */
		add_filter('pll_preferred_language', array( $this, 'set_preferred_language'), 10, 2);
    }

    /**
     * Filter the visitor's preferred language (normally set first by cookie
     * if this is not the first visit, then by the country).
     * If no preferred language has been found or set by this filter,
     * Polylang fallbacks to the default language
     *
     * @param string|bool $language Preferred language code, false if none has been found.
     * @param bool        $cookie   Whether the preferred language has been defined by the cookie.
     */
    public function set_preferred_language( $language, $cookie )
    {
        // Check first if the user was already browsing this site.
        if( $cookie ) {
            return $language;
        }

        // Browser language detetion is disabled
        if( !PLL()->options['browser'] ) {
            return $language;
        }

        // Get the preferred languages according to the IP address
        $accept_langs = $this->get_accept_country_langs();

        if( empty($accept_langs) ) {
            return $language;
        }

        $languages = PLL()->model->get_languages_list( array( 'hide_empty' => true ) ); // Hides languages with no post

        // Filter the list of languages to use to match the country
        $languages = apply_filters( 'pll_languages_for_browser_preferences', $languages );

        // Looks through sorted list and use first one that matches our language list
        foreach ( array_keys( $accept_langs ) as $accept_lang )
        {
            // First loop to match the exact locale
            foreach ( $languages as $language ) {
                if ( 0 === strcasecmp( $accept_lang, $language->get_locale() ) ) {
                    return $language->slug;
                }
            }

            // Second loop to match the language set
            foreach ( $languages as $language ) {
                if ( 0 === stripos( $accept_lang, $language->slug ) || 0 === stripos( $language->get_locale(), $accept_lang ) ) {
                    return $language->slug;
                }
            }
        }

        return $language;
    }

    /**
	 * Get the preferred languages according to the IP address
	 *
	 * @return array the preferred language slugs
	 */
    private function get_accept_country_langs()
    {
        $accept_langs = array();

        // Get the remote IP address
        $ip_address = $_SERVER['REMOTE_ADDR'];

        // Determines if the IP address is valid.
        if( !rest_is_ip_address( $ip_address ) ) {
            return $accept_langs;
        }

        $url = add_query_arg( array(
            'lang'      => 'en',
            'fields'    => 'status,message,countryCode'
        ), 'http://ip-api.com/json/' . $ip_address );

        // Do a IP address search by the IP Geolocation API (https://ip-api.com/)
        $request = wp_remote_get( $url );

        // Check if there was an request error
        if( is_wp_error( $request ) || wp_remote_retrieve_response_message($request) !== 'OK' ) {
            return $accept_langs;
        }

        // Retrieve the body from the response
        $api_data = json_decode( wp_remote_retrieve_body( $request ), true );

        // IP Geolocation API response was successfully
        if ( isset( $api_data['status'], $api_data['countryCode'] ) && $api_data['status'] === 'success' )
        {
            // Get all accept languages by country code (flag)
            $accept_langs = wp_list_filter( PLL_Settings::get_predefined_languages(), array(
                'flag' => strtolower( $api_data['countryCode'] )
            ) );
        }

        return $accept_langs;
    }

    /**
	 * Get the preferred languages according to the browser preferences
	 * Code adapted from http://www.thefutureoftheweb.com/blog/use-accept-language-header
	 *
	 * @return array the preferred language slugs
	 */
    private function get_accept_browser_langs()
    {
        $accept_langs = array();

		if ( isset( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ) {
			// Break up string into pieces ( languages and q factors )
			preg_match_all(
				'/([a-z]{1,8}(-[a-z]{1,8})?)\s*(;\s*q\s*=\s*((?>1|0)(?>\.[0-9]+)?))?/i',
				sanitize_text_field( wp_unslash( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ),
				$lang_parse
			);

			$k = $lang_parse[1];
			$v = $lang_parse[4];

			if ( $n = count( $k ) ) {
				// Set default to 1 for any without q factor
				foreach ( $v as $key => $val ) {
					if ( '' === $val || (float) $val > 1 ) {
						$v[ $key ] = 1;
					}
				}

				// Bubble sort ( need a stable sort for Android, so can't use a PHP sort function )
				if ( $n > 1 ) {
					for ( $i = 2; $i <= $n; $i++ ) {
						for ( $j = 0; $j <= $n - 2; $j++ ) {
							if ( $v[ $j ] < $v[ $j + 1 ] ) {
								// Swap values
								$temp = $v[ $j ];
								$v[ $j ] = $v[ $j + 1 ];
								$v[ $j + 1 ] = $temp;
								// Swap keys
								$temp = $k[ $j ];
								$k[ $j ] = $k[ $j + 1 ];
								$k[ $j + 1 ] = $temp;
							}
						}
					}
				}
				$accept_langs = array_combine( $k, $v );
			}
		}

		return wp_list_filter( $accept_langs, array( '0' ), 'NOT' ); // Remove languages markes as unacceptable (q=0).
    }
}

new Polylang_Country_Detection();